#include "hello_view.h"
#include "app.h"
#include <assert.h>

#include <ion.h>
#include <kandinsky.h>
#include <escher.h>
#include <stdlib.h>
#include <stdint.h>
#include <stddef.h>

#include <python/port/port.h>

extern "C" {
  #include <python/port/mphalport.h>
}

#define API_VERSION 1

uint64_t millis() {
  return Ion::Timing::millis();
}

void msleep(uint32_t ms) {
  Ion::Timing::msleep(ms);
}

uint64_t scanKeyboard() {
  return Ion::Keyboard::scan();
}

void pushRect(int16_t x, int16_t y, uint16_t w, uint16_t h, const uint16_t * pixels) {
  KDRect rect(x, y, w, h);

  Ion::Display::pushRect(rect, reinterpret_cast<const KDColor*>(pixels));
}

void pushRectUniform(int16_t x, int16_t y, uint16_t w, uint16_t h, uint16_t color) {
  KDRect rect(x, y, w, h);

  Ion::Display::pushRectUniform(rect, KDColor::RGB16(color));
}

int16_t drawTextLarge(const char *text, int16_t x, int16_t y, uint16_t fg, uint16_t bg) {
  KDPoint point(x, y);

  auto ctx = KDIonContext::sharedContext();
  ctx->setClippingRect(KDRect(0,0,320,240));
  ctx->setOrigin(KDPoint(0,0));
  ctx->drawString(text, point, KDFont::LargeFont, KDColor::RGB16(fg), KDColor::RGB16(bg));

  return point.x();
}

int16_t drawTextSmall(const char *text, int16_t x, int16_t y, uint16_t fg, uint16_t bg) {
  KDPoint point(x, y);

  auto ctx = KDIonContext::sharedContext();
  ctx->setClippingRect(KDRect(0,0,320,240));
  ctx->setOrigin(KDPoint(0,0));
  ctx->drawString(text, point, KDFont::SmallFont, KDColor::RGB16(fg), KDColor::RGB16(bg));

  return point.x();
}

void waitForVBlank() {
  Ion::Display::waitForVBlank();
}

void clipboardStore(const char *text) {
  Clipboard::sharedClipboard()->store(text);
}

const char * clipboardText() {
  return Clipboard::sharedClipboard()->storedText();
}

void (* const apiPointers[])(void) = {
  (void (*)(void)) millis,
  (void (*)(void)) msleep,
  (void (*)(void)) scanKeyboard,
  (void (*)(void)) pushRect,
  (void (*)(void)) pushRectUniform,
  (void (*)(void)) drawTextLarge,
  (void (*)(void)) drawTextSmall,
  (void (*)(void)) waitForVBlank,
  (void (*)(void)) clipboardStore,
  (void (*)(void)) clipboardText,
  (void (*)(void)) nullptr,
  (void (*)(void)) nullptr,
  (void (*)(void)) nullptr,
  (void (*)(void)) nullptr,
  (void (*)(void)) nullptr,
  (void (*)(void)) nullptr,
  (void (*)(void)) nullptr,
  (void (*)(void)) nullptr,
  (void (*)(void)) nullptr,
  (void (*)(void)) nullptr,
  (void (*)(void)) nullptr,
  (void (*)(void)) nullptr,
  (void (*)(void)) nullptr,
  (void (*)(void)) nullptr,
  (void (*)(void)) nullptr,
  (void (*)(void)) nullptr,
  (void (*)(void)) nullptr,
  (void (*)(void)) nullptr,
  (void (*)(void)) nullptr,
  (void (*)(void)) nullptr,
  (void (*)(void)) nullptr,
  (void (*)(void)) nullptr,
  (void (*)(void)) nullptr,
  (void (*)(void)) nullptr,
  (void (*)(void)) nullptr,
  (void (*)(void)) nullptr,
  (void (*)(void)) nullptr,
  (void (*)(void)) nullptr,
  (void (*)(void)) nullptr,
  (void (*)(void)) nullptr,
  (void (*)(void)) nullptr,
  (void (*)(void)) nullptr,
  (void (*)(void)) nullptr,
  (void (*)(void)) nullptr,
  (void (*)(void)) nullptr,
  (void (*)(void)) nullptr,
  (void (*)(void)) nullptr,
  (void (*)(void)) nullptr,
  (void (*)(void)) nullptr,
  (void (*)(void)) nullptr,
  (void (*)(void)) nullptr,
  (void (*)(void)) nullptr,
  (void (*)(void)) nullptr,
  (void (*)(void)) nullptr,
  (void (*)(void)) nullptr,
  (void (*)(void)) nullptr,
  (void (*)(void)) nullptr,
  (void (*)(void)) nullptr,
  (void (*)(void)) nullptr,
  (void (*)(void)) nullptr,
  (void (*)(void)) nullptr,
  (void (*)(void)) nullptr,
  (void (*)(void)) nullptr,
  (void (*)(void)) nullptr,

  // khicas from here, subject to change
  (void (*)(void)) numworks_draw_string,
  (void (*)(void)) numworks_draw_string_small,
  (void (*)(void)) numworks_set_pixel,
  (void (*)(void)) numworks_fill_rect,
  (void (*)(void)) numworks_get_pixel,
  (void (*)(void)) numworks_hide_graph,
  (void (*)(void)) numworks_wait_1ms,
  (void (*)(void)) waitforvblank,
  (void (*)(void)) statuslinemsg,
  (void (*)(void)) statusline,
  (void (*)(void)) getkey,
  (void (*)(void)) GetKey,
  (void (*)(void)) alphawasactive,
  (void (*)(void)) lock_alpha,
  (void (*)(void)) reset_kbd,
  (void (*)(void)) back_key_pressed,
  (void (*)(void)) enable_back_interrupt,
  (void (*)(void)) disable_back_interrupt,
  (void (*)(void)) os_set_angle_unit,
  (void (*)(void)) os_get_angle_unit,
  (void (*)(void)) os_file_browser,
  (void (*)(void)) file_exists,
  (void (*)(void)) erase_file,
  (void (*)(void)) read_file,
  (void (*)(void)) write_file,
  (void (*)(void)) mp_hal_input,
  (void (*)(void)) nullptr
};

const void * getApiPointers() {
  return apiPointers;
}

struct TarHeader
{                              /* byte offset */
  char name[100];               /*   0 */
  char mode[8];                 /* 100 */
  char uid[8];                  /* 108 */
  char gid[8];                  /* 116 */
  char size[12];                /* 124 */
  char mtime[12];               /* 136 */
  char chksum[8];               /* 148 */
  char typeflag;                /* 156 */
  char linkname[100];           /* 157 */
  char magic[8];                /* 257 */
  char uname[32];               /* 265 */
  char gname[32];               /* 297 */
  char devmajor[8];             /* 329 */
  char devminor[8];             /* 337 */
  char padding[167];            /* 345 */
} __attribute__((packed));

static_assert(sizeof(TarHeader) == 512);

bool isSane(const TarHeader* tar) {
  return !memcmp(tar->magic, "ustar  ", 8) && tar->name[0] != '\x00' && tar->name[0] != '\xFF';
}

constexpr int MaxNameLength = 40;
int indexFromName(const char *name) ;

struct File {
    const char *name;
    const uint8_t *data;
    size_t dataLength;
    bool isExecutable;
};

bool fileAtIndex(size_t index, File &entry) {
  const TarHeader* tar = reinterpret_cast<const TarHeader*>(0x90200000);
  unsigned size = 0;

  // Sanity check.
  if (!isSane(tar)) {
    return false;
  }

  /**
   * TAR files are comprised of a set of records aligned to 512 bytes boundary
   * followed by data.
   */
  while (index-- > 0) {
    size = 0;
    for (int i = 0; i < 11; i++)
      size = size * 8 + (tar->size[i] - '0');

    // Move to the next TAR header.
    unsigned stride = (sizeof(TarHeader) + size + 511);
    stride = (stride >> 9) << 9;
    tar = reinterpret_cast<const TarHeader*>(reinterpret_cast<const char*>(tar) + stride);

    // Sanity check.
    if (!isSane(tar)) {
      return false;
    }
  }

  // File entry found, copy data out.
  entry.name = tar->name;
  entry.data = reinterpret_cast<const uint8_t*>(tar) + sizeof(TarHeader);
  entry.dataLength = size;
  entry.isExecutable = (tar->mode[4] & 0x01) == 1;

  return true;
}

typedef uint32_t (*entrypoint)(const uint32_t, const void *, void *, const uint32_t);

uint32_t executeFile(const char *name, void * heap, const uint32_t heapSize) {
  File entry;
  if(fileAtIndex(indexFromName(name), entry)) {
    if(!entry.isExecutable) {
      return 0;
    }
    uint32_t ep = *reinterpret_cast<const uint32_t*>(entry.data);
    if(ep >= 0x90200000 && ep < 0x90800000) {
      return ((entrypoint)ep)(API_VERSION, getApiPointers(), heap, heapSize);
    }
  }
  return -1;
}

int indexFromName(const char *name) {
  File entry;

  for (int i = 0; fileAtIndex(i, entry); i++) {
    if (strcmp(name, entry.name) == 0) {
      return i;
    }
  }

  return -1;
}

size_t numberOfFiles() {
  File dummy;
  size_t count;

  for (count = 0; fileAtIndex(count, dummy); count++);

  return count;
}

namespace Hello {

  HelloView::HelloView(::App * app) :
  View()
{
  m_app=app;
}

void HelloView::drawRect(KDContext * ctx, KDRect rect) const {
  executeFile("KhiCAS",((App *)m_app)->heap(), ((App *)m_app)->heapsize());
  ctx->fillRect(bounds(), KDColorWhite);
  ctx->drawString("   HOME: Menu principal", KDPoint(0,0));
  ctx->drawString("De la, HOME lance l'extension", KDPoint(0,18));
  int y=54;
  ctx->drawString("  Delta (c) 2019 par Damien", KDPoint(0,y)); y+=18;
  ctx->drawString("Nicolet et Bernard Parisse.", KDPoint(0,y)); y+=18;
  ctx->drawString("Squelette Hello par M. Friess", KDPoint(0,y)); y+=18;
  ctx->drawString("Epsilon (c) 2019 Numworks", KDPoint(0,y)); y+=18;
  ctx->drawString("   Licence CC-NC-SA   ", KDPoint(0,y)); y+=18;
  ctx->drawString("Merci a Jean-Baptiste Boric,", KDPoint(0,y)); y+=18;
  ctx->drawString("M. Friess et a l'equipe tiplanet", KDPoint(0,y)); y+=18;
  ctx->drawString("pour l'aide apportee au portage", KDPoint(0,y)); y+=18;
  ctx->drawString("de Khicas sur Numworks.", KDPoint(0,y)); y+=18;

}

}
